clc; clear;
% Inputs
N=100000; % Number of simulations
r=1;    % Radius of hemisphere

x=random('Uniform',0,1,1,N);
y=random('Uniform',0,1,1,N);

figure % Plot generated pseudorandom numbers - x,y <0..1,0..1>
plot(x,y,'r.');
axis equal;
xlabel('x');
ylabel('y');
title('Generated points with x, y coordinates - range 0..1');

for i=1:1:N
  xr(i)=-r+2*r*x(i);
  yr(i)=-r+2*r*y(i);
end

figure % Plot generated pseudorandom numbers - x,y <-r..+r,-r..+r>
theta=0:0.001:2*pi;     % set of angles for the circle
circlex=+r*cos(theta); %function to create the x values of the circle. 
                       %circle is centered at (0,0) and has a radius of r
circley=+r*sin(theta); %function to create y values of circle
mask = xr.^2 + yr.^2 <= r^2;
ax = axes();
hold(ax);
scatter(xr(mask),yr(mask), 'b.')
scatter(xr(~mask),yr(~mask), 'g.')
scatter(circlex,circley,'r.');
hold off; %plot the circle inside of the square
% plot(xr,yr,'b.');
axis equal;
xlabel('x');
ylabel('y');
title('Generated points with x, y coordinates - range -r..+r');

for i=1:1:N % z coordinates calculation
  if (xr(i)^2+yr(i)^2<=r^2)
     zr(i)=sqrt(r^2-(xr(i)^2+yr(i)^2)); 
  else
     zr(i)=0; 
  end
end

figure % Plot generated points
ax = axes();
hold(ax);
scatter3(xr(mask),yr(mask),zr(mask),'b.')
scatter3(xr(~mask),yr(~mask),zr(~mask),'g.')
view(-45,20)
%plot3(xr,yr,zr,'k.');
axis equal;
xlabel('x');
ylabel('y');
zlabel('f(x,y)');
title('Generated points with x, y coordinates and value of f(x,y)');

Integral=((2*r)^2)*sum(zr)/(N);   % Hemisphere volume calculated using Monte Carlo
Int_real=2/3*pi*r^3;              % Hemisphere volume, exact solution 

Mean_val=sum(zr)/(N);

for i=1:1:N
  dist(i)=(zr(i)-Mean_val)^2;
end

Standard_dev=sqrt(sum(dist)/(N+1)); % Standard deviation of the Monte Carlo calculation
Err=Standard_dev/sqrt(N+1)*(2*r)^2; % Error estimation of the calculated integral

disp(sprintf('\n'))
disp('*****************************************************************')
disp('* Calculation of the hemisphere volume using Monte Carlo method *')
disp('*****************************************************************')
disp(' ')
disp('Inputs:')
disp('--------------------------------------------------------------')
disp(sprintf('Number of simulations N                       = %10.0f',N));
disp(' ')
disp('Results:')
disp('--------------------------------------------------------------')
disp(sprintf('Hemisphere volume using Monte Carlo method    = %10.6f m^3',Integral));
disp(sprintf('Exact value of hemisphere volume              = %10.6f m^3',Int_real));
disp(sprintf('Difference between the exact and MC results   = %10.6f m^3',abs(Integral-Int_real)));
disp(sprintf('Difference between the exact and MC results   = %7.3f%%',abs(Integral-Int_real)/Integral*100));
disp(sprintf('Error estimation of the calculated integral   = %10.6f m^3',Err));
disp(sprintf('Error estimation of the calculated integral   = %7.3f%%',Err/Integral*100));